@@ -10,7 +10,7 @@
   See the accompanying LICENSE file.
 */
 
-/** 
+/**
 
 .. module:: apsw
    :synopsis: Python access to SQLite database library
@@ -44,6 +44,7 @@ API Reference
 #define SQLITE_API static
 #define SQLITE_EXTERN static
 #include APSW_USE_SQLITE_AMALGAMATION
+#undef small
 
 /* Fight with SQLite over ndebug */
 #ifdef APSW_NO_NDEBUG
@@ -57,8 +58,8 @@ API Reference
 #include "sqlite3.h"
 #endif
 
-#if SQLITE_VERSION_NUMBER < 3007006
-#error Your SQLite version is too old.  It must be at least 3.7.6
+#if SQLITE_VERSION_NUMBER < 3007016
+#error Your SQLite version is too old.  It must be at least 3.7.16
 #endif
 
 /* system headers */
@@ -116,7 +117,7 @@ static PyObject *apswmodule;
 #ifdef PYPY_VERSION
 #include "pypycompat.c"
 #endif
- 
+
 /* Augment tracebacks */
 #include "traceback.c"
 
@@ -171,6 +172,21 @@ getsqliteversion(void)
   return MAKESTR(sqlite3_libversion());
 }
 
+/** .. method:: sqlite3_sourceid() -> string
+
+    Returns the exact checkin information for the SQLite 3 source
+    being used.
+
+    -* sqlite3_sourceid
+*/
+
+static PyObject *
+get_sqlite3_sourceid(void)
+{
+  return MAKESTR(sqlite3_sourceid());
+}
+
+
 /** .. method:: apswversion() -> string
 
   Returns the APSW version.
@@ -186,9 +202,9 @@ getapswversion(void)
   If you use the same :class:`Connection` across threads or use
   multiple :class:`connections <Connection>` accessing the same file,
   then SQLite can `share the cache between them
-  <http://www.sqlite.org/sharedcache.html>`_.  It is :ref:`not
+  <https://sqlite.org/sharedcache.html>`_.  It is :ref:`not
   recommended <sharedcache>` that you use this.
-  
+
   -* sqlite3_enable_shared_cache
 */
 static PyObject *
@@ -244,7 +260,7 @@ static PyObject *
 sqliteshutdown(void)
 {
   int res;
-  
+
   APSW_FAULT_INJECT(ShutdownFail, res=sqlite3_shutdown(), res=SQLITE_NOMEM);
   SET_EXC(res, NULL);
 
@@ -256,13 +272,14 @@ sqliteshutdown(void)
 
 /** .. method:: config(op[, *args])
 
-  :param op: A `configuration operation <http://sqlite.org/c3ref/c_config_chunkalloc.html>`_
+  :param op: A `configuration operation <https://sqlite.org/c3ref/c_config_chunkalloc.html>`_
   :param args: Zero or more arguments as appropriate for *op*
 
-  Many operations don't make sense from a Python program.  Only the
+  Many operations don't make sense from a Python program.  The
   following configuration operations are supported: SQLITE_CONFIG_LOG,
   SQLITE_CONFIG_SINGLETHREAD, SQLITE_CONFIG_MULTITHREAD,
-  SQLITE_CONFIG_SERIALIZED and SQLITE_CONFIG_MEMSTATUS.
+  SQLITE_CONFIG_SERIALIZED, SQLITE_CONFIG_URI, SQLITE_CONFIG_MEMSTATUS
+  and SQLITE_CONFIG_COVERING_INDEX_SCAN.
 
   See :func:`log` for an example of using SQLITE_CONFIG_LOG.
 
@@ -272,7 +289,7 @@ sqliteshutdown(void)
 #ifdef EXPERIMENTAL
 static PyObject *logger_cb=NULL;
 
-static void 
+static void
 apsw_logger(void *arg, int errcode, const char *message)
 {
   PyGILState_STATE gilstate;
@@ -291,7 +308,7 @@ apsw_logger(void *arg, int errcode, const char *message)
   if(!res)
     {
       AddTraceBackHere(__FILE__, __LINE__, "Call_Logger",
-		       "{s: O, s: i, s: s}", 
+		       "{s: O, s: i, s: s}",
 		       "logger", arg,
 		       "errcode", errcode,
 		       "message", message);
@@ -324,13 +341,15 @@ config(APSW_ARGUNUSED PyObject *self, PyObject *args)
     case SQLITE_CONFIG_SINGLETHREAD:
     case SQLITE_CONFIG_MULTITHREAD:
     case SQLITE_CONFIG_SERIALIZED:
+    case SQLITE_CONFIG_URI:
       if(!PyArg_ParseTuple(args, "i", &optdup))
         return NULL;
       assert(opt==optdup);
       res=sqlite3_config( (int)opt );
       break;
-      
+
     case SQLITE_CONFIG_MEMSTATUS:
+    case SQLITE_CONFIG_COVERING_INDEX_SCAN:
       {
         int boolval;
         if(!PyArg_ParseTuple(args, "ii", &optdup, &boolval))
@@ -339,7 +358,7 @@ config(APSW_ARGUNUSED PyObject *self, PyObject *args)
         res=sqlite3_config( (int)opt, boolval);
         break;
       }
-      
+
     case SQLITE_CONFIG_LOG:
       {
 	PyObject *logger;
@@ -488,10 +507,10 @@ releasememory(APSW_ARGUNUSED PyObject *self, PyObject *args)
 
   Returns current and highwater measurements.
 
-  :param op: A `status parameter <http://sqlite.org/c3ref/c_status_malloc_size.html>`_
+  :param op: A `status parameter <https://sqlite.org/c3ref/c_status_malloc_size.html>`_
   :param reset: If *True* then the highwater is set to the current value
   :returns: A tuple of current value and highwater value
-  
+
   .. seealso::
 
     * :ref:`Status example <example-status>`
@@ -532,7 +551,7 @@ vfsnames(APSW_ARGUNUSED PyObject *self)
 
   while(vfs)
     {
-      APSW_FAULT_INJECT(vfsnamesfails, 
+      APSW_FAULT_INJECT(vfsnamesfails,
                         str=convertutf8string(vfs->zName),
                         str=PyErr_NoMemory());
       if(!str) goto error;
@@ -552,11 +571,11 @@ vfsnames(APSW_ARGUNUSED PyObject *self)
 
   If you would like to raise an exception that corresponds to a
   particular SQLite `error code
-  <http://sqlite.org/c3ref/c_abort.html>`_ then call this function.
+  <https://sqlite.org/c3ref/c_abort.html>`_ then call this function.
   It also understands `extended error codes
-  <http://sqlite.org/c3ref/c_ioerr_access.html>`_.
+  <https://sqlite.org/c3ref/c_ioerr_access.html>`_.
 
-  For example to raise `SQLITE_IOERR_ACCESS <http://sqlite.org/c3ref/c_ioerr_access.html>`_::
+  For example to raise `SQLITE_IOERR_ACCESS <https://sqlite.org/c3ref/c_ioerr_access.html>`_::
 
     raise apsw.exceptionfor(apsw.SQLITE_IOERR_ACCESS)
 
@@ -640,7 +659,7 @@ apsw_test_reset_rng(APSW_ARGUNUSED PyObject *self)
 #ifdef APSW_TESTFIXTURES
 /* xGetLastError isn't actually called anywhere by SQLite so add a
    manual way of doing so
-   http://www.sqlite.org/cvstrac/tktview?tn=3337 */
+   https://sqlite.org/cvstrac/tktview?tn=3337 */
 
 static PyObject *
 apsw_call_xGetLastError(APSW_ARGUNUSED PyObject *self, PyObject *args)
@@ -681,168 +700,10 @@ apsw_fini(APSW_ARGUNUSED PyObject *self)
 }
 #endif
 
-#ifdef APSW_USE_SQLITE_ASYNCVFS_H
-
-#define SQLITE3ASYNC_API static
-
-#include APSW_USE_SQLITE_ASYNCVFS_H
-
-/** .. method:: async_initialize(parentvfs, makedefault) -> string
-
-  Registers the asyncvfs.  It is safe to call this method multiple
-  times, with subsequent calls having no effect.
-
-  :param parentvfs: Name of the vfs to use for I/O operations.  Use an empty string for the default
-  :param makedefault: Make the async vfs be the default vfs.
-  :returns: A string containg the name of the default vfs.
-
-  Note that this method cannot return string errors as part of
-  exceptions, just generic error codes and messages.  For example if
-  you specify a non-existent parentvfs then you just get
-  :exc:`apsw.SQLError` with the text `error`.  The `source
-  <http://www.sqlite.org/src/finfo?name=ext/async/sqlite3async.h>`__
-  lists what error codes could be returned.
-
-  .. seealso::
-
-  * :ref:`ext-asyncvfs`
-*/
-static PyObject *
-apsw_async_initialize(APSW_ARGUNUSED PyObject *self, PyObject *args)
-{
-  char *vfsname=NULL;
-  int makedefault=0;
-  int res;
-
-  if(!PyArg_ParseTuple(args, "esi", STRENCODING, &vfsname, &makedefault))
-    return NULL;
-
-  if(!*vfsname)
-    {
-      PyMem_Free(vfsname);
-      vfsname=NULL;
-    }
-
-  _PYSQLITE_CALL_V(res=sqlite3async_initialize(vfsname, makedefault));
-  PyMem_Free(vfsname);
-  if(res!=SQLITE_OK)
-    {
-      SET_EXC(res, NULL);
-      return NULL;
-    }
-
-  return convertutf8string(SQLITEASYNC_VFSNAME);
-}
-
-/** .. method:: async_shutdown()
-
-  Unregisters the asyncvfs.  It is safe to call this method multiple
-  times with subsequent calls having no effect.  It is not recommended
-  that you call this method.  It deallocates everything immediately
-  making no checks if anything is in use.  If you want to call it then
-  only do so once you are absolutely sure that all databases, files,
-  vfs, threads and other objects it references have been closed and
-  freed.
-*/
-static PyObject *
-apsw_async_shutdown(APSW_ARGUNUSED PyObject *self)
-{
-  _PYSQLITE_CALL_V(sqlite3async_shutdown());
-
-  Py_RETURN_NONE;
-}
-
-/** .. method:: async_control(op[, params])
-
-    :param op: The operation code (eg SQLITE_ASYNC_HALT)
-    :param params: Optional additional parameters depending on the opcode.
-    :returns: None or additional information requested
-
-    This table summarises the parameters taken and value returned
-    depending on the operation.
-
-    +-----------------------------------+--------------+---------+
-    | op                                | params       | result  |
-    +===================================+==============+=========+
-    | SQLITEASYNC_HALT                  | int          | None    |
-    +-----------------------------------+--------------+---------+
-    | SQLITEASYNC_GET_HALT              |              | int     |
-    +-----------------------------------+--------------+---------+
-    | SQLITEASYNC_DELAY                 | int          | None    |
-    +-----------------------------------+--------------+---------+
-    | SQLITEASYNC_GET_DELAY             |              | int     |
-    +-----------------------------------+--------------+---------+
-    | SQLITEASYNC_LOCKFILES             | int          | None    |
-    +-----------------------------------+--------------+---------+
-    | SQLITEASYNC_GET_LOCKFILES         |              | int     |
-    +-----------------------------------+--------------+---------+
-*/
-static PyObject *
-apsw_async_control(APSW_ARGUNUSED PyObject *self, PyObject *args)
-{
-  int op, inparam, outparam,res;
-
-  if(!PyTuple_Check(args) || PyTuple_GET_SIZE(args)<1)
-    return PyErr_Format(PyExc_TypeError, "Args should be a tuple of at least one item");
-
-  if(!PyIntLong_Check(PyTuple_GET_ITEM(args, 0)))
-    return PyErr_Format(PyExc_TypeError, "Arg 0 must be a number");
-
-  switch(PyIntLong_AsLong(PyTuple_GET_ITEM(args, 0)))
-    {
-    case SQLITEASYNC_HALT:
-    case SQLITEASYNC_DELAY:
-    case SQLITEASYNC_LOCKFILES:
-      if(!PyArg_ParseTuple(args, "ii", &op, &inparam))
-	return NULL;
-      _PYSQLITE_CALL_V(res=sqlite3async_control(op, inparam));
-      if(res!=SQLITE_OK)
-	{
-	  SET_EXC(res, NULL);
-	  return NULL;
-	}
-      Py_RETURN_NONE;
-
-    case SQLITEASYNC_GET_HALT:
-    case SQLITEASYNC_GET_DELAY:
-    case SQLITEASYNC_GET_LOCKFILES:
-      op=PyIntLong_AsLong(PyTuple_GET_ITEM(args, 0));
-      assert(!PyErr_Occurred());
-      _PYSQLITE_CALL_V(res=sqlite3async_control(op, &outparam));
-      APSW_FAULT_INJECT(AsyncControlFails,,res=SQLITE_NOMEM);
-      if(res!=SQLITE_OK)
-	{
-	  SET_EXC(res, NULL);
-	  return NULL;
-	}
-      return PyInt_FromLong(outparam);
-
-    default:
-      return PyErr_Format(PyExc_ValueError, "Unknown operation argument");
-    }
-}
-
-/** .. method:: async_run()
-
-  Call this method from a worker thread and it will do all the
-  asynchronous I/O.  You can get the function to exit by calling
-  :func:`async_control` with a first parameter of :const:`SQLITEASYNC_HALT` and
-  a second parameter of :const:`SQLITEASYNC_HALT_NOW`.
-*/
-static PyObject *
-apsw_async_run(APSW_ARGUNUSED PyObject *self)
-{
-  _PYSQLITE_CALL_V(sqlite3async_run());
-
-  Py_RETURN_NONE;
-}
-
-
-#endif
 
 #ifdef APSW_FORK_CHECKER
 
-/* 
+/*
    We want to verify that SQLite objects are not used across forks.
    One way is to modify all calls to SQLite to do the checking but
    this is a pain as well as a performance hit.  Instead we use the
@@ -851,7 +712,7 @@ apsw_async_run(APSW_ARGUNUSED PyObject *self)
 
    Our diverted functions check the process id on calls and set the
    process id on allocating a mutex.  We have to avoid the checks for
-   the static mutexes.  
+   the static mutexes.
 
    This code also doesn't bother with some things like checking malloc
    results.  It is intended to only be used to verify correctness with
@@ -860,7 +721,7 @@ apsw_async_run(APSW_ARGUNUSED PyObject *self)
    good thing - you will really be sure there is a problem!
  */
 
-typedef struct 
+typedef struct
 {
   pid_t pid;
   sqlite3_mutex *underlying_mutex;
@@ -905,7 +766,7 @@ apsw_xMutexAlloc(int which)
       {
 	apsw_mutex *am;
 	sqlite3_mutex *m=apsw_orig_mutex_methods.xMutexAlloc(which);
-	
+
 	if(!m) return m;
 
 	am=malloc(sizeof(apsw_mutex));
@@ -1019,7 +880,7 @@ static sqlite3_mutex_methods apsw_mutex_methods=
 
   SQLite does not allow the use of database connections across `forked
   <http://en.wikipedia.org/wiki/Fork_(operating_system)>`__ processes
-  (see the `SQLite FAQ Q6 <http://www.sqlite.org/faq.html#q6>`__).
+  (see the `SQLite FAQ Q6 <https://sqlite.org/faq.html#q6>`__).
   (Forking creates a child process that is a duplicate of the parent
   including the state of all data structures in the program.  If you
   do this to SQLite then parent and child would both consider
@@ -1073,10 +934,10 @@ apsw_fork_checker(APSW_ARGUNUSED PyObject *self)
   /* then do a shutdown as we can't get or change mutex while sqlite is running */
   rc=sqlite3_shutdown();
   if(rc) goto fail;
-  
+
   rc=sqlite3_config(SQLITE_CONFIG_GETMUTEX, &apsw_orig_mutex_methods);
   if(rc) goto fail;
-  
+
   rc=sqlite3_config(SQLITE_CONFIG_MUTEX, &apsw_mutex_methods);
   if(rc) goto fail;
 
@@ -1086,7 +947,7 @@ apsw_fork_checker(APSW_ARGUNUSED PyObject *self)
 
  ok:
   Py_RETURN_NONE;
-  
+
  fail:
   assert(rc!=SQLITE_OK);
   SET_EXC(rc, NULL);
@@ -1173,6 +1034,7 @@ formatsqlvalue(APSW_ARGUNUSED PyObject *self, PyObject *value)
       Py_ssize_t left;
       unires=PyUnicode_FromUnicode(NULL, PyUnicode_GET_SIZE(value)+2);
       if(!unires) return NULL;
+      APSW_UNICODE_READY(unires, return NULL);
       res=PyUnicode_AS_UNICODE(unires);
       *res++='\'';
       memcpy(res, PyUnicode_AS_UNICODE(value), PyUnicode_GET_DATA_SIZE(value));
@@ -1202,9 +1064,9 @@ formatsqlvalue(APSW_ARGUNUSED PyObject *self, PyObject *value)
 	      if(*res==0)
 		{
 		  *res++='\'';
-		  *res++='|'; *res++='|'; 
-		  *res++='X'; *res++='\''; *res++='0'; *res++='0'; *res++='\''; 
-		  *res++='|'; *res++='|'; 
+		  *res++='|'; *res++='|';
+		  *res++='X'; *res++='\''; *res++='0'; *res++='0'; *res++='\'';
+		  *res++='|'; *res++='|';
 		  *res='\'';
 		}
 	      else
@@ -1227,7 +1089,7 @@ formatsqlvalue(APSW_ARGUNUSED PyObject *self, PyObject *value)
       int asrb;
       PyObject *unires;
       Py_UNICODE *res;
-#define _HEXDIGITS 
+#define _HEXDIGITS
 
       asrb=PyObject_AsReadBuffer(value, (const void**)&buffer, &buflen);
       APSW_FAULT_INJECT(FormatSQLValueAsReadBufferFails,
@@ -1239,8 +1101,9 @@ formatsqlvalue(APSW_ARGUNUSED PyObject *self, PyObject *value)
       APSW_FAULT_INJECT(FormatSQLValuePyUnicodeFromUnicodeFails,
 			unires=PyUnicode_FromUnicode(NULL, buflen*2+3),
 			unires=PyErr_NoMemory());
-      if(!unires) 
+      if(!unires)
 	return NULL;
+      APSW_UNICODE_READY(unires, return NULL);
       res=PyUnicode_AS_UNICODE(unires);
       *res++='X';
       *res++='\'';
@@ -1301,6 +1164,8 @@ apsw_log(APSW_ARGUNUSED PyObject *self, PyObject *args)
 
 
 static PyMethodDef module_methods[] = {
+  {"sqlite3_sourceid", (PyCFunction)get_sqlite3_sourceid, METH_NOARGS,
+   "Return the source identification of the SQLite library"},
   {"sqlitelibversion", (PyCFunction)getsqliteversion, METH_NOARGS,
    "Return the version of the SQLite library"},
   {"apswversion", (PyCFunction)getapswversion, METH_NOARGS,
@@ -1347,16 +1212,6 @@ static PyMethodDef module_methods[] = {
   {"_fini", (PyCFunction)apsw_fini, METH_NOARGS,
    "Frees all caches and recycle lists"},
 #endif
-#ifdef APSW_USE_SQLITE_ASYNCVFS_H
-  {"async_initialize", (PyCFunction)apsw_async_initialize, METH_VARARGS,
-   "Initializes the asyncvfs extension"},
-  {"async_shutdown", (PyCFunction)apsw_async_shutdown, METH_NOARGS,
-   "Frees all asyncvfs objects"},
-  {"async_control", (PyCFunction)apsw_async_control, METH_VARARGS,
-   "Control operation of asyncvfs"},
-  {"async_run", (PyCFunction)apsw_async_run, METH_NOARGS,
-   "Does the background async I/O"},
-#endif
 #ifdef APSW_FORK_CHECKER
   {"fork_checker", (PyCFunction)apsw_fork_checker, METH_NOARGS,
    "Installs fork checking code"},
@@ -1371,7 +1226,7 @@ static void add_shell(PyObject *module);
 #if PY_MAJOR_VERSION >= 3
 static struct PyModuleDef apswmoduledef={
   PyModuleDef_HEAD_INIT,
-  "apsw", 
+  "apsw",
   NULL,
   -1,
   module_methods,
@@ -1385,7 +1240,7 @@ static struct PyModuleDef apswmoduledef={
 
 PyMODINIT_FUNC
 #if PY_MAJOR_VERSION < 3
-initapsw(void) 
+initapsw(void)
 #else
 PyInit_apsw(void)
 #endif
@@ -1412,6 +1267,7 @@ PyInit_apsw(void)
         || PyType_Ready(&APSWBlobType) <0
         || PyType_Ready(&APSWVFSType) <0
         || PyType_Ready(&APSWVFSFileType) <0
+	|| PyType_Ready(&APSWURIFilenameType) <0
         || PyType_Ready(&APSWStatementType) <0
         || PyType_Ready(&APSWBufferType) <0
         || PyType_Ready(&FunctionCBInfoType) <0
@@ -1432,16 +1288,16 @@ PyInit_apsw(void)
 #endif
 
     if (m == NULL)  goto fail;
-    
+
     Py_INCREF(m);
 
     if(init_exceptions(m)) goto fail;
 
     Py_INCREF(&ConnectionType);
     PyModule_AddObject(m, "Connection", (PyObject *)&ConnectionType);
-    
+
     /* we don't add cursor, blob or backup to the module since users shouldn't be able to instantiate them directly */
-    
+
     Py_INCREF(&ZeroBlobBindType);
     PyModule_AddObject(m, "zeroblob", (PyObject *)&ZeroBlobBindType);
 
@@ -1449,7 +1305,10 @@ PyInit_apsw(void)
     PyModule_AddObject(m, "VFS", (PyObject*)&APSWVFSType);
     Py_INCREF(&APSWVFSFileType);
     PyModule_AddObject(m, "VFSFile", (PyObject*)&APSWVFSFileType);
-    
+    Py_INCREF(&APSWURIFilenameType);
+    PyModule_AddObject(m, "URIFilename", (PyObject*)&APSWURIFilenameType);
+
+
     /** .. attribute:: connection_hooks
 
        The purpose of the hooks is to allow the easy registration of
@@ -1480,16 +1339,16 @@ PyInit_apsw(void)
 
     */
     PyModule_AddIntConstant(m, "SQLITE_VERSION_NUMBER", SQLITE_VERSION_NUMBER);
-    
+
 
     /** .. attribute:: using_amalgamation
 
     If True then `SQLite amalgamation
-    <http://www.sqlite.org/cvstrac/wiki?p=TheAmalgamation>`__ is in
+    <https://sqlite.org/cvstrac/wiki?p=TheAmalgamation>`__ is in
     use (statically compiled into APSW).  Using the amalgamation means
     that SQLite shared libraries are not used and will not affect your
     code.
-    
+
     */
 
 #ifdef APSW_USE_SQLITE_AMALGAMATION
@@ -1500,7 +1359,7 @@ PyInit_apsw(void)
     PyModule_AddObject(m, "using_amalgamation", Py_False);
 #endif
 
-    /** 
+    /**
 
 .. _sqliteconstants:
 
@@ -1508,7 +1367,7 @@ SQLite constants
 ================
 
 SQLite has `many constants
-<http://sqlite.org/c3ref/constlist.html>`_ used in various
+<https://sqlite.org/c3ref/constlist.html>`_ used in various
 interfaces.  To use a constant such as :const:`SQLITE_OK`, just
 use ``apsw.SQLITE_OK``.
 
@@ -1543,7 +1402,7 @@ modules etc. For example::
       ADDINT(SQLITE_IGNORE),
       ADDINT(SQLITE_OK),
       END,
-      
+
       DICT("mapping_authorizer_function"),
       ADDINT(SQLITE_CREATE_INDEX),
       ADDINT(SQLITE_CREATE_TABLE),
@@ -1615,6 +1474,28 @@ modules etc. For example::
       ADDINT(SQLITE_IOERR_SHMOPEN),
       ADDINT(SQLITE_IOERR_SHMSIZE),
       ADDINT(SQLITE_IOERR_SHMLOCK),
+      ADDINT(SQLITE_CORRUPT_VTAB),
+      ADDINT(SQLITE_IOERR_SEEK),
+      ADDINT(SQLITE_IOERR_SHMMAP),
+      ADDINT(SQLITE_READONLY_CANTLOCK),
+      ADDINT(SQLITE_READONLY_RECOVERY),
+      ADDINT(SQLITE_ABORT_ROLLBACK),
+      ADDINT(SQLITE_CANTOPEN_ISDIR),
+      ADDINT(SQLITE_CANTOPEN_FULLPATH),
+      ADDINT(SQLITE_IOERR_DELETE_NOENT),
+      ADDINT(SQLITE_CONSTRAINT_CHECK),
+      ADDINT(SQLITE_CONSTRAINT_COMMITHOOK),
+      ADDINT(SQLITE_CONSTRAINT_FOREIGNKEY),
+      ADDINT(SQLITE_CONSTRAINT_FUNCTION),
+      ADDINT(SQLITE_CONSTRAINT_NOTNULL),
+      ADDINT(SQLITE_CONSTRAINT_PRIMARYKEY),
+      ADDINT(SQLITE_CONSTRAINT_TRIGGER),
+      ADDINT(SQLITE_CONSTRAINT_UNIQUE),
+      ADDINT(SQLITE_CONSTRAINT_VTAB),
+      ADDINT(SQLITE_READONLY_ROLLBACK),
+      ADDINT(SQLITE_IOERR_MMAP),
+      ADDINT(SQLITE_NOTICE_RECOVER_ROLLBACK),
+      ADDINT(SQLITE_NOTICE_RECOVER_WAL),
       END,
 
       /* error codes */
@@ -1646,6 +1527,8 @@ modules etc. For example::
       ADDINT(SQLITE_NOTADB),
       ADDINT(SQLITE_NOTFOUND),
       ADDINT(SQLITE_TOOBIG),
+      ADDINT(SQLITE_NOTICE),
+      ADDINT(SQLITE_WARNING),
       END,
 
       /* open flags */
@@ -1668,6 +1551,8 @@ modules etc. For example::
       ADDINT(SQLITE_OPEN_SHAREDCACHE),
       ADDINT(SQLITE_OPEN_AUTOPROXY),
       ADDINT(SQLITE_OPEN_WAL),
+      ADDINT(SQLITE_OPEN_URI),
+      ADDINT(SQLITE_OPEN_MEMORY),
       END,
 
       /* limits */
@@ -1683,7 +1568,7 @@ modules etc. For example::
       ADDINT(SQLITE_LIMIT_LIKE_PATTERN_LENGTH),
       ADDINT(SQLITE_LIMIT_VARIABLE_NUMBER),
       ADDINT(SQLITE_LIMIT_TRIGGER_DEPTH),
-      /* We don't include the MAX limits - see http://code.google.com/p/apsw/issues/detail?id=17 */
+      /* We don't include the MAX limits - see https://code.google.com/p/apsw/issues/detail?id=17 */
       END,
 
       DICT("mapping_config"),
@@ -1702,6 +1587,12 @@ modules etc. For example::
       ADDINT(SQLITE_CONFIG_LOG),
       ADDINT(SQLITE_CONFIG_GETPCACHE),
       ADDINT(SQLITE_CONFIG_PCACHE),
+      ADDINT(SQLITE_CONFIG_URI),
+      ADDINT(SQLITE_CONFIG_PCACHE2),
+      ADDINT(SQLITE_CONFIG_GETPCACHE2),
+      ADDINT(SQLITE_CONFIG_COVERING_INDEX_SCAN),
+      ADDINT(SQLITE_CONFIG_SQLLOG),
+      ADDINT(SQLITE_CONFIG_MMAP_SIZE),
       END,
 
       DICT("mapping_db_config"),
@@ -1732,6 +1623,9 @@ modules etc. For example::
       ADDINT(SQLITE_DBSTATUS_LOOKASIDE_HIT),
       ADDINT(SQLITE_DBSTATUS_LOOKASIDE_MISS_FULL),
       ADDINT(SQLITE_DBSTATUS_LOOKASIDE_MISS_SIZE),
+      ADDINT(SQLITE_DBSTATUS_CACHE_HIT),
+      ADDINT(SQLITE_DBSTATUS_CACHE_MISS),
+      ADDINT(SQLITE_DBSTATUS_CACHE_WRITE),
       END,
 
       DICT("mapping_locking_level"),
@@ -1761,6 +1655,7 @@ modules etc. For example::
       ADDINT(SQLITE_IOCAP_SAFE_APPEND),
       ADDINT(SQLITE_IOCAP_SEQUENTIAL),
       ADDINT(SQLITE_IOCAP_UNDELETABLE_WHEN_OPEN),
+      ADDINT(SQLITE_IOCAP_POWERSAFE_OVERWRITE),
       END,
 
       DICT("mapping_sync"),
@@ -1784,28 +1679,20 @@ modules etc. For example::
       ADDINT(SQLITE_FCNTL_CHUNK_SIZE),
       ADDINT(SQLITE_FCNTL_FILE_POINTER),
       ADDINT(SQLITE_FCNTL_SYNC_OMITTED),
+      ADDINT(SQLITE_FCNTL_PERSIST_WAL),
+      ADDINT(SQLITE_FCNTL_WIN32_AV_RETRY),
+      ADDINT(SQLITE_FCNTL_OVERWRITE),
+      ADDINT(SQLITE_FCNTL_POWERSAFE_OVERWRITE),
+      ADDINT(SQLITE_FCNTL_VFSNAME),
+      ADDINT(SQLITE_FCNTL_PRAGMA),
+      ADDINT(SQLITE_FCNTL_BUSYHANDLER),
+      ADDINT(SQLITE_FCNTL_TEMPFILENAME),
+      ADDINT(SQLITE_FCNTL_MMAP_SIZE),
       END
 
-#ifdef APSW_USE_SQLITE_ASYNCVFS_H
-      ,
-      DICT("mapping_asyncvfs_control"),
-      ADDINT(SQLITEASYNC_HALT),
-      ADDINT(SQLITEASYNC_GET_HALT),
-      ADDINT(SQLITEASYNC_DELAY),
-      ADDINT(SQLITEASYNC_GET_DELAY),
-      ADDINT(SQLITEASYNC_LOCKFILES),
-      ADDINT(SQLITEASYNC_GET_LOCKFILES),
-      END,
-
-      DICT("mapping_asyncvfs_control_halt"),
-      ADDINT(SQLITEASYNC_HALT_NEVER),
-      ADDINT(SQLITEASYNC_HALT_NOW),
-      ADDINT(SQLITEASYNC_HALT_IDLE),
-      END
-#endif
       };
- 
- 
+
+
  for(i=0;i<sizeof(integers)/sizeof(integers[0]); i++)
    {
      const char *name=integers[i].name;
@@ -1861,7 +1748,7 @@ modules etc. For example::
 
  fail:
     Py_XDECREF(m);
-    return 
+    return
 #if PY_MAJOR_VERSION >= 3
           NULL
 #endif
@@ -1911,9 +1798,9 @@ APSW_Should_Fault(const char *name)
     PyObject_SetAttrString(apswmodule, "faultdict", PyDict_New());
 
   value=MAKESTR(name);
-  
+
   faultdict=PyObject_GetAttrString(apswmodule, "faultdict");
-  
+
   truthval=PyDict_GetItem(faultdict, value);
   if(!truthval)
     goto finally;
@@ -1930,9 +1817,3 @@ APSW_Should_Fault(const char *name)
   return res;
 }
 #endif
-
-/* async vfs */
-#ifdef APSW_USE_SQLITE_ASYNCVFS_C
-#define SQLITE_ENABLE_ASYNCIO
-#include APSW_USE_SQLITE_ASYNCVFS_C
-#endif
